__author__ = "Richard Mwewa"
__about__ = "https://about.me/rly0nheart"
__version__ = "1.1.1"

__description__ = """# Orbis Unum
> 🌍 IP Geolocator & Coordinate Mapper 📍"""
__epilog__ = f"# by [{__author__}]({__about__})"
